; *****************************
; ** angelZware PB Tools Lib **
; ** ----------------------- **
; ** awpop3.pbi              **
; ** V0.3 (2010/08/26)       **
; ** by Ronny Krger         **
; *****************************
; ** Licence: LGPL V3        **
; *****************************

XIncludeFile "awnetwork.pbi"
XIncludeFile "awsupport.pbi"

CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
  XIncludeFile "awopenssl.pbi"
CompilerEndIf

;**
;* awpop3.pbi_
;* Version 0.3_
;* _
;* POP3-Client fr PureBasic 4.50+_
;* Lizenz: LGPL V3_
;* _
;* Systemanforderungen:_
;* <table style="text-align: left;" border="0" cellpadding="2" cellspacing="0"><tbody><tr>
;* <td style="vertical-align: top;">Windows</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Linux</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Mac</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">x86</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">x64</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">PPC</td><td style="vertical-align: top;">[ ]</td></tr><tr>
;* <td style="vertical-align: top;">ASCII</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;">Unicode</td><td style="vertical-align: top;">[x]</td>
;* <td style="vertical-align: top;"></td><td style="vertical-align: top;"></td></tr></tbody></table>_
;* Wichtig: Vor der Benutzung muss einmalig InitNetwork() aufgerufen werden!

;{ private functions
;{ structures
Structure ____POP3_Conn Extends ____NW_NetworkConn
  Caps.s
  TimeStamp.s
  ; offen fr Erweiterungen
EndStructure
;}
;{ Constants
#____POP3_Connect_APOP  = $0001
#____POP3_Connect_USER  = $0002
#____POP3_Connect_AUTO  = $0003
;}
;{ functions
Procedure.i ____AllocatePOP3(TimeOut.i = 15000, Retries.i = 5)
  Protected *pop.____POP3_Conn
  
  *pop = AllocateMemory(SizeOf(____POP3_Conn))      ; Speicher fr die POP3-Struktur anfordern
  If *pop
    InitializeStructure(*pop, ____POP3_Conn)        ; Datenstruktur initialisieren
    *pop\TimeOut      = TimeOut
    *pop\Retries      = Retries
    *pop\CallbackFunc = #Null
    *pop\Caps         = ""
    *pop\TimeStamp    = ""
    *pop\Buffer       = AllocateMemory(#____NW_BUF_SIZE + 2)    ; Sende- und Empfangspuffer alloziieren; maximale Gre ca 65000 Bytes
    If Not *pop\Buffer
      FreeMemory(*pop)
      *pop = #Null
    EndIf
  EndIf
  
  ProcedureReturn *pop
EndProcedure
Procedure   ____FreePOP3(*pop.____POP3_Conn)
  If *pop
    If *pop\Buffer
      FreeMemory(*pop\Buffer)     ; Sende/Empfangspuffer freigeben
    EndIf
    FreeMemory(*pop)              ; Datenstruktur freigeben
  EndIf
EndProcedure

Procedure.b ____POP3NoOp(*pop.____POP3_Conn)              ; POP3-Befehlt NOOP senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *pop\Retries
  Repeat
    res = ____NW_Execute(*pop, "NOOP", *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____POP3Rset(*pop.____POP3_Conn)              ; POP3-Befehlt RSET senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *pop\Retries
  Repeat
    res = ____NW_Execute(*pop, "RSET", *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True
        Case #____NW_CD_ERR:  res = #False : retries - 1
       Default:              res = #False
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____POP3Dele(*pop.____POP3_Conn, index.i)     ; POP3-Befehlt DELE senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *pop\Retries
  Repeat
    res = ____NW_Execute(*pop, "DELE " + Str(index), *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.s ____POP3Capa(*pop.____POP3_Conn)              ; POP3-Befehlt CAPA senden und Ergebnis auswerten
  Protected res.b, retries.i, result.s
  
  retries = *pop\Retries
  result  = ""
  Repeat
    res = ____NW_Execute(*pop, "CAPA", *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True  : ____NW_ReadData(*pop) : result = *pop\DataString
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False : result = ""
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn result
EndProcedure
Procedure.i ____POP3Stat(*pop.____POP3_Conn)              ; POP3-Befehlt STAT senden und Ergebnis auswerten
  Protected res.b, retries.i, result.i
  
  retries = *pop\Retries
  result  = -1
  Repeat
    res = ____NW_Execute(*pop, "STAT", *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True  : result = Val(StringField(*pop\ResultString, 2, " "))
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False : result = -1
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn result
EndProcedure
Procedure.s ____POP3Top(*pop.____POP3_Conn, index.i)      ; POP3-Befehlt TOP senden und Ergebnis auswerten
  Protected res.b, retries.i, result.s
  
  retries = *pop\Retries
  result  = ""
  Repeat
    res = ____NW_Execute(*pop, "TOP " + Str(index) + " 0", *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True  : ____NW_ReadData(*pop) : result = *pop\DataString
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False : result = ""
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn result
EndProcedure
Procedure.s ____POP3Retr(*pop.____POP3_Conn, index.i)     ; POP3-Befehlt RETR senden und Ergebnis auswerten
  Protected res.b, retries.i, result.s
  
  retries = *pop\Retries
  result  = ""
  Repeat
    res = ____NW_Execute(*pop, "RETR " + Str(index), *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True  : ____NW_ReadData(*pop) : result = *pop\DataString
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False : result = ""
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn result
EndProcedure
Procedure.s ____POP3List(*pop.____POP3_Conn, index.i = 0) ; POP3-Befehlt LIST senden und Ergebnis auswerten
  Protected res.b, retries.i, result.s
  
  retries = *pop\Retries
  result  = ""
  Repeat
    If index
      res = ____NW_Execute(*pop, "LIST " + Str(index), *pop\Retries)
    Else
      res = ____NW_Execute(*pop, "LIST", *pop\Retries)     
    EndIf
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True  : result = *pop\ResultString
         Case #____NW_CD_ERR:  res = #False : retries - 1
       Default:              res = #False : result = ""
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn result
EndProcedure
Procedure.b ____POP3Quit(*pop.____POP3_Conn)              ; POP3-Befehlt QUIT senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = *pop\Retries
  Repeat
    res = ____NW_Execute(*pop, "QUIT", *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____POP3User(*pop.____POP3_Conn)              ; POP3-Befehlt USER senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = 1; *pop\Retries
  Repeat
    res = ____NW_Execute(*pop, "USER " + *pop\user, *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____POP3Pass(*pop.____POP3_Conn)              ; POP3-Befehlt PASS senden und Ergebnis auswerten
  Protected res.b, retries.i
  
  retries = 1; *pop\Retries
  Repeat
    res = ____NW_Execute(*pop, "PASS " + *pop\pass, *pop\Retries)
    If res
      Select *pop\ResultCode
        Case #____NW_CD_OK:   res = #True
        Case #____NW_CD_ERR:  res = #False : retries - 1
        Default:              res = #False
      EndSelect
    EndIf
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure.b ____POP3AuthLogin(*pop.____POP3_Conn)         ; Login via USER/PASS
  If ____POP3User(*pop)
    If ____POP3Pass(*pop)
      ProcedureReturn #True
    EndIf
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.b ____POP3AuthAPOP(*pop.____POP3_Conn)          ; Login via APOP
  Protected res.b, digest.s, retries.i
  res = #False
  If *pop\TimeStamp
    digest  = ____SUP_MD5Str(*pop\TimeStamp + *pop\pass)
    retries = *pop\Retries
    Repeat
      res = ____NW_Execute(*pop, "APOP " + *pop\user + " " + digest, *pop\Retries)
      If res
        Select *pop\ResultCode
          Case #____NW_CD_OK:   res = #True
          Case #____NW_CD_ERR:  res = #False : retries - 1
          Default:              res = #False
        EndSelect
      EndIf
    Until res Or (retries <= 0)
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____POP3StartTLS(*pop.____POP3_Conn)
  Protected res.b
  
  res     = #False
  
  CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
    retries = *pop\Retries
    Repeat
      res = ____NW_Execute(*pop, "STLS", *pop\Retries)
      If res
        Select *pop\ResultCode
          Case #____NW_CD_OK:   res = #True
          Case #____NW_CD_ERR:  res = #False : retries - 1
          Default:              res = #False
        EndSelect
      EndIf
    Until res Or (retries <= 0)
    
    If res
      res = ____SSL_DoConnect(*pop)
    EndIf
  CompilerEndIf
  
  ProcedureReturn res
EndProcedure

Procedure.b ____POP3Connect(*pop.____POP3_Conn, mode.i, autotls.b)   ; mit POP3-Server verbinden und einloggen
  Protected res.b, tsb.i, tse.i, res2.b
  
  res         = #False
  *pop\ConnID = OpenNetworkConnection(*pop\Host, *pop\Port)
  If *pop\ConnID
    If ____NW_ReadResult(*pop) = #____NW_CD_OK
      tsb = FindString(*pop\ResultString, "<", 1)                 ; Timestamp aus Serverbegrung extrahieren; ggf fr APOP ntig
      If tsb
        tse = ____MSG_FindLastChar(*pop\ResultString, '>')
        If tse > tsb
          *pop\TimeStamp = Mid(*pop\ResultString, tsb, tse - tsb + 1)
        EndIf
      EndIf
      *pop\caps = ____POP3Capa(*pop)
      CompilerIf Defined(Enable_OpenSSL, #PB_Constant)
        If autotls And FindString(*pop\caps, "STLS", 1)                 ; TLS starten; OpenSSL erforderlich [hierzu #Enable_OpenSSL VOR include der datei setzen]
          *smtp\caps = ""
          res        = ____POP3StartTLS(*pop)
          If res
            *pop\caps = ____POP3Capa(*pop)
          EndIf
        EndIf
      CompilerEndIf
      
      res2 = #False
      If (mode & #____POP3_Connect_APOP) And FindString(*pop\caps, "APOP", 1)
        res  = ____POP3AuthAPOP(*pop)
        res2 = #True
      EndIf
      If Not res And (mode & #____POP3_Connect_USER) And FindString(*pop\caps, "USER", 1)
        res  = ____POP3AuthLogin(*pop)
        res2 = #True
      EndIf
      If Not res2 And (mode & #____POP3_Connect_AUTO)
        res = ____POP3Stat(*pop)      ; z.B. fr T-Online ntig, da der Benutzer ber den Anschluss authentifiziert wird; in dem Falle funktioniert STAT ohne Login
      EndIf
    EndIf
    If Not res
      CloseNetworkConnection(*pop\ConnID)
      *pop\ConnID = #Null
    EndIf
  EndIf
  
  ProcedureReturn res
EndProcedure
Procedure.b ____POP3DisConnect(*pop.____POP3_Conn)        ; POP3-Verbindung trennen
  If ____POP3Quit(*pop)
    CloseNetworkConnection(*pop\ConnID)
    *pop\ConnID = #Null
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure

Procedure.s ____POP3GetServerResult(*pop.____POP3_Conn)   ; Resultstring zur Fehlerausgabe zurckgeben
  If *pop
    ProcedureReturn *pop\ResultString
  EndIf
  
  ProcedureReturn ""
EndProcedure
;}
;}
;{ public functions
;{ Constants
#AWPB_POP3_Connect_USER  = #____POP3_Connect_USER                       ;* USER/PASS benutzen
#AWPB_POP3_Connect_APOP  = #____POP3_Connect_APOP                       ;* APOP benutzen
#AWPB_POP3_Connect_AUTO  = #____POP3_Connect_AUTO                       ;* Verbindungsmethode automatisch bestimmen
;}
;** awpb_pop3_allocatepop3
;* Fordert Speicher fr ein POP3-Objekt an._
;* Rckgabewert ist *POP3Object. Ist der Rckgabewert #Null, wurde kein Objekt angelegt.
Procedure.i AWPB_POP3_AllocatePOP3()
  ProcedureReturn ____AllocatePOP3()
EndProcedure
;** awpb_pop3_freepop3
;* Gibt ein POP3-Objekt frei.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
Procedure   AWPB_POP3_FreePOP3(*POP3Object)
  ____FreePOP3(*POP3Object)
EndProcedure

;** awpb_pop3_setserver
;* Setzt die Serverdaten des Objektes.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .HostName.s: Name des Servers
;** .Port.w: Portnummer
Procedure.b AWPB_POP3_SetServer(*POP3Object, HostName.s, Port.w)
  Protected *pop.____POP3_Conn
  
  *pop = *POP3Object
  If *pop
    *pop\Host = HostName
    *pop\Port = Port
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure
;** awpb_pop3_setlogin
;* Setzt die Logindaten des Objektes.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .User.s: Benutzername
;** .Pass.s: Passwort
Procedure.b AWPB_POP3_SetLogin(*POP3Object, User.s, Pass.s)
  Protected *pop.____POP3_Conn
  
  *pop = *POP3Object
  If *pop
    *pop\User = User
    *pop\Pass = Pass
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure

;** awpb_pop3_connect
;* Stellt eine Verbindung zum Server her. Vorher mssen mit AWPB_POP3_SetServer() und AWPB_POP3_SetLogin() die Logindaten gesetzt werden._
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .mode: Verbindungsmodus; einer dieser Werte:_
;* <ul><li><b>#AWPB_POP3_Connect_AUTO</b></li><li>#AWPB_POP3_Connect_USER</li><li>#AWPB_POP3_Connect_APOP</li></ul>
Procedure.b AWPB_POP3_Connect(*POP3Object, mode.i = #AWPB_POP3_Connect_AUTO)
  ProcedureReturn ____POP3Connect(*POP3Object, mode, #False)
EndProcedure
;** awpb_pop3_disconnect
;* Trennt die Verbindung mit dem Server_
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*POP3Object: Zeiger auf ein POP3-Objekt
Procedure.b AWPB_POP3_DisConnect(*POP3Object)
  ProcedureReturn ____POP3DisConnect(*POP3Object)
EndProcedure

;** awpb_pop3_noop
;* bermittelt dem Server den Befehl NOOP._
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*POP3Object: Zeiger auf ein POP3-Objekt
Procedure.b AWPB_POP3_NOOP(*POP3Object)
  ProcedureReturn ____POP3NoOp(*POP3Object)
EndProcedure
;** awpb_pop3_rset
;* bermittelt dem Server den Befehl RSET._
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*POP3Object: Zeiger auf ein POP3-Objekt
Procedure.b AWPB_POP3_RSET(*POP3Object)
  ProcedureReturn ____POP3Rset(*POP3Object)
EndProcedure
;** awpb_pop3_dele
;* bermittelt dem Server den Befehl DELE. Die Mail mit dem bergebenen Index wird gelscht._
;* Rckgabewert ist bei Erfolg #True, im Fehlerfall #False
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .Index.i: Index der zu lschenden Nachricht
Procedure.b AWPB_POP3_DELE(*POP3Object, Index.i)
  ProcedureReturn ____POP3Dele(*POP3Object, Index)
EndProcedure
;** awpb_pop3_stat
;* bermittelt dem Server den Befehl STAT._
;* Rckgabewert ist bei Erfolg die Anzahl der Mails auf dem Server, im Fehlerfalle -1.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
Procedure.i AWPB_POP3_STAT(*POP3Object)
  ProcedureReturn ____POP3Stat(*POP3Object)
EndProcedure
;** awpb_pop3_top
;* bermittelt dem Server den Befehl TOP. Hiermit wird der Header der angegebenen Nachricht abgerufen und zurckgegeben._
;* Im Fehlerfall wird ein leerer String zurckgegeben.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .Index.i: Index der abzurufenden Nachricht
Procedure.s AWPB_POP3_TOP(*POP3Object, Index.i)
  ProcedureReturn ____POP3Top(*POP3Object, Index)
EndProcedure
;** awpb_pop3_retr
;* bermittelt dem Server den Befehl RETR. Hiermit wird die angegebene Nachricht abgerufen und zurckgegeben._
;* Im Fehlerfall wird ein leerer String zurckgegeben.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .Index.i: Index der abzurufenden Nachricht
Procedure.s AWPB_POP3_RETR(*POP3Object, Index.i)
  ProcedureReturn ____POP3Retr(*POP3Object, Index)
EndProcedure
;** awpb_pop3_list
;* bermittelt dem Server den Befehl LIST. Die abgerufene Liste wird als String zurckgegeben.; Wird ein Index > 0 bergeben, 
;* werden nur die Informationen dieser einen Nachricht abgerufen._
;* Im Fehlerfall wird ein leerer String zurckgegeben.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .Index.i: Index der Nachricht oder 0 fr alle
Procedure.s AWPB_POP3_LIST(*POP3Object, Index.i = 0)
  ProcedureReturn ____POP3List(*POP3Object, Index)
EndProcedure

;** awpb_pop3_getserverresult
;* Gibt den letzten vom Server erhaltenen String zurck, z.B. zur Ausgabe einer Fehlermeldung durch das Programm.
;** .*POP3Object: Zeiger auf ein SMTP-Objekt
Procedure.s AWPB_POP3_GetServerResult(*POP3Object)
  ProcedureReturn ____POP3GetServerResult(*POP3Object)
EndProcedure
;** awpb_pop3_setcallback
;* Setzt eine Callback-Funktion, die an verschiedenen Stellen automatisch aufgerufen wird._ 
;* Die Callback-Funktion muss folgendem Muster entsprechen: <b>Procedure.b CallbackFunction(StatusText.s)</b>._
;* Der Prozedur wird ein String mit dem aktuellen Status bergeben. Ist der Rckgabewert der Funktion #False, wird die POP3-Operation abgebrochen._
;* Wird als Zeiger #Null bergeben, wird die gespeicherte Prozedur entfernt.
;** .*POP3Object: Zeiger auf ein POP3-Objekt
;** .*CallbackFuncPointer: Zeiger auf eine Callbackfunktion oder #Null
Procedure   AWPB_POP3_SetCallback(*POP3Object, *CallbackFuncPointer = #Null)
  ____NW_SetCallbackFunc(*POP3Object, *CallbackFuncPointer)
EndProcedure
;}
   
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 365
; FirstLine = 118
; Folding = ZAwLAAw
; EnableUnicode
; EnableThread
; EnableXP
; CPU = 1